@@ -341,11 +341,12 @@ ice <- function(data, time_points, id, time_name,
   clean_kwarg_str <- paste0(str_remove_all(as.character(deparse(sub_kwarg)), " "), collapse = "")
   kwarg_list <- as.list(sub_kwarg)
   kwarg_name_list <- names(kwarg_list)
-  threshold_idx <- str_which(str_split(as.character(substitute(list(...))), " = "), "threshold")
-  static_idx <- str_which(str_split(as.character(substitute(list(...))), " = "), "static")
-  dynamic_idx <- str_which(str_split(as.character(substitute(list(...))), " = "), "dynamic")
-  grace_period_idx <- str_which(str_split(as.character(substitute(list(...))), " = "), "grace_period")
-  nc_idx <- str_which(str_split(as.character(substitute(list(...))), " = "), "natural_course")
+
+  threshold_idx <- which(sapply(kwarg_list, function(x) any(str_detect(as.character(x), "threshold"))))
+  static_idx <- which(sapply(kwarg_list, function(x) any(str_detect(as.character(x), "static"))))
+  dynamic_idx <- which(sapply(kwarg_list, function(x) any(str_detect(as.character(x), "dynamic"))))
+  grace_period_idx <- which(sapply(kwarg_list, function(x) any(str_detect(as.character(x), "grace_period"))))
+  nc_idx <- which(sapply(kwarg_list, function(x) any(str_detect(as.character(x), "natural_course"))))
   
   ## preprocess static intervention
   
@@ -428,8 +429,7 @@ ice <- function(data, time_points, id, time_name,
   
   clean_kwarg_str <- preprocess_dynamic$clean_kwarg_str
   kwarg_list <- preprocess_dynamic$kwarg_list
-  
-  # print(clean_kwarg_str)
+
   ## get argument list
   args <- c(as.list(environment()), eval(parse(text = clean_kwarg_str)))
   kwargs_len <- length(eval(parse(text = clean_kwarg_str)))
@@ -583,11 +583,13 @@ ice <- function(data, time_points, id, time_name,
     interv_var_single <- list()
     interv_time_single <- list()
 
+
     for (treat in 1:length(interv_idx)) {
 
       treat_idx <- interv_idx[treat]
       treat_name <- paste0("intervention", interv_list_origin[treat_idx])
       interv <- as.list(arg_interv[[which(names(arg_interv) == treat_name)]])[[1]]
+      
       interv_var <- interv_all_names[treat]
       des <- as.list(int_descript[i])
       if (length(arg_interv[[which(names(arg_interv) == treat_name)]]) == 2) {
@@ -596,6 +598,7 @@ ice <- function(data, time_points, id, time_name,
         times <- 0:(K-1)
       }
 
+
       interv_single <- c(interv_single, list(interv))
       interv_var_single <- c(interv_var_single, list(interv_var))
       interv_time_single <- c(interv_time_single, list(times))
@@ -1727,6 +1730,9 @@ construct_interv_value <- function(data, timevar, int_value, int_time, int_var){
 
   for (i in 1:nint) {
     this_int_value <- int_value[[i]]
+    if (is.data.frame(this_int_value)) {
+      this_int_value <- this_int_value[, "interv_values"]
+    }
     not_int_idx <- which(!data[, timevar] %in% int_time[[i]])
     
     if (length(not_int_idx) > 0) {
